
# q-expansion of newform 8820.2.a.r, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8820
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [0], [-3], [1], [-3], [-2], [6], [9], [-8], [-10], [0], [2], [-3], [0], [12], [-8], [8], [0], [-14], [5], [-12], [12], [-17], [-6], [7], [6], [-19], [6], [20], [-18], [-12], [-2], [-18], [-19], [22], [2], [9], [-3], [-12], [-8], [-3], [-4], [-12], [-20], [5], [-17], [-27], [16], [-24], [-9], [10], [6], [-6], [30], [18], [16], [-22], [21], [-11], [-15], [13], [18], [13], [18], [20], [2], [-30], [10], [-15], [0], [-5], [-4], [-4], [36], [-9], [-5], [15], [-26], [-24], [29], [-15], [-14], [10], [6], [-15], [-28], [0], [8], [15], [6], [-10], [15], [-25], [-33], [18], [-30], [-20], [-25], [8], [0], [-12], [6], [-4], [-47], [-24], [-33], [9], [22], [1], [2], [-30], [10], [-25], [30], [-41], [48], [6], [33], [-8], [20], [15], [-24], [28], [-3], [-1], [-18], [-32], [13], [11], [-48], [53], [-16], [-42], [-2], [15], [-5], [-3], [-9], [34], [15], [-40], [42], [40], [18], [22], [6], [4], [-12], [-22], [48], [20], [-48], [-22], [0], [41], [-24], [7], [0], [-36], [-36], [-34], [36], [18], [-33], [-16], [-17], [29], [-42], [-24], [-14], [-27], [62], [46], [-36], [-25], [-9], [-20], [16], [-4], [60], [-10], [42], [36], [-54], [20], [31], [-31], [24], [34], [6], [-7], [-12], [60], [39], [-1], [-64], [45], [33], [-6], [22], [-14], [10], [30], [57], [34], [-12], [42], [40], [32], [18], [-64], [21], [18], [-68], [-22], [24], [42], [-42], [-22], [-2], [51], [-34], [-3], [-10], [57], [9], [-41], [36], [68], [-38], [8], [-9], [-5], [57], [-44], [-24], [-57], [6], [0], [64], [-5], [-34], [-9], [30], [-56], [42], [77], [-42], [-22], [-42], [-48], [16], [-30], [57], [-34], [28], [63], [-47], [26], [6], [16], [-11], [-66], [10], [15], [78], [-40], [30], [-68], [56], [-17], [44], [22], [-29], [36], [2], [17], [-6], [33], [77], [48], [-38], [67], [9], [14], [-66], [-32], [0], [39], [-33], [-12], [0], [-22], [60], [-62], [-9], [-60], [31], [67], [-42], [-61], [-72], [65], [-10], [78], [40], [-24], [68], [0], [21], [-78], [50], [30], [4], [90], [-39], [-5], [75], [-38], [-52], [18], [8], [18], [-8], [41], [76], [-12], [24], [-34], [-48], [94], [51], [-76], [-63], [-43], [-18], [46], [43], [-10], [12], [90], [35], [6], [36], [82], [-28], [78], [-51], [-68], [56], [15], [19], [17], [-30], [30], [18], [30], [-18], [80], [60], [-60], [9], [28], [-46], [69], [62], [-43], [-15], [-56], [-36], [-3], [-62], [2], [48], [69], [25], [54], [37], [18], [6], [-16], [-90], [76], [-39], [-49], [49], [-76], [51], [-31], [33], [-21], [-92], [57], [-40], [-20], [-36], [11], [78], [-68], [-24], [-22], [-87], [84], [-74], [-10], [57], [-44], [-12], [-17], [18], [42], [-28], [-22], [-72], [-39], [100], [30], [-15], [3], [22], [66], [60], [-11], [45], [-60], [30], [-20], [-24], [-2], [-48], [-76], [-45], [37], [60], [-25], [-25], [80], [-20], [-45], [-87], [26], [-105], [-44], [-90], [-59], [-36], [88], [-62], [68], [-24], [12], [60], [-82], [-9], [-19], [-84], [-77], [30], [-61], [71], [111], [71], [13], [-52], [-16], [-69], [18], [-44], [44], [66], [-72], [17], [105], [-89], [-42], [4], [0], [90], [22], [102], [25], [24], [1], [-54], [50], [90], [-98], [-16], [-35], [3], [65], [-93], [-15], [-68], [43], [-6], [94], [53], [-30], [13], [24], [2], [86], [-99], [-72], [-2], [8], [-44], [12], [-114], [-74], [48], [-61], [-40], [-60], [-44], [-75], [-68], [86], [77], [-114], [78], [10], [90], [-38], [-39], [-90], [12], [-100], [48], [-40], [75], [-11], [81], [-77], [0], [-31], [38], [-78], [60], [28], [-3], [51], [53], [32], [12], [-11], [6], [-63], [-35], [117], [24], [36], [52], [-4]]
